<?php

namespace Drupal\layout_builder_ipe\Validation\Constraint;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\Plugin\Validation\Constraint\EntityChangedConstraint;
use Drupal\Core\Entity\Plugin\Validation\Constraint\EntityChangedConstraintValidator;
use Drupal\layout_builder_ipe\LayoutBuilderIpeService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates the EntityChanged constraint.
 */
class LayoutBuilderEntityChangedConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * The entity decorator service.
   *
   * @var \Drupal\layout_builder_ipe\LayoutBuilderIpeService
   */
  protected $layoutBuilderIpe;

  /**
   * Creates an EntityLabelNotNullConstraintValidator object.
   *
   * @param \Drupal\layout_builder_ipe\LayoutBuilderIpeService $layout_builder_ipe
   *   The layout builder IPE service.
   */
  public function __construct(LayoutBuilderIpeService $layout_builder_ipe) {
    $this->layoutBuilderIpe = $layout_builder_ipe;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('layout_builder_ipe'));
  }

  /**
   * {@inheritdoc}
   */
  public function validate($entity, Constraint $constraint) {
    if (isset($entity)) {
      /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
      if (!$entity->isNew()) {
        $saved_entity = \Drupal::entityTypeManager()->getStorage($entity->getEntityTypeId())->loadUnchanged($entity->id());
        if ($this->layoutBuilderIpe->ipeEnabled($entity)) {
          // This is an entity that uses IPE.
          if ($this->layoutBuilderIpe->isLayoutBuilderIpeFormSubmission($entity)) {
            // This is a form submission from the layout builder form. All that
            // can be changed there is the layout, so all we need to check is if
            // the layout has changed.
            if ($this->hasLayoutChanges($saved_entity)) {
              // The layout has changed, so we add a violation.
              $this->context->addViolation($constraint->message);
            }
          }
          else {
            // This is some other modification to the entity, presumably a
            // submission from the node edit form.
            if ($this->hasEntityChanges($saved_entity)) {
              // The layout has changed, so we add a violation.
              $entity_changed_constraint = new EntityChangedConstraint();
              $this->context->addViolation($entity_changed_constraint->message);
            }
          }
        }
        else {
          // This is an entity that doesn't use IPE, so we fall back to the
          // original EntityChanged constraint.
          $original_validator = new EntityChangedConstraintValidator();
          $original_validator->initialize($this->context);
          $original_validator->validate($entity, new EntityChangedConstraint);
        }
      }
    }
  }

  /**
   * See if the given entity has been submitted via Layout Builder IPE.
   *
   * @param \Drupal\Core\Entity\EntityInterface
   *   The entity object.
   *
   * @return bool
   */
  public function hasLayoutChanges($entity) {
    $original_hash = \Drupal::request()->get('layout_builder_ipe_layout_hash');
    if (empty($original_hash)) {
      return FALSE;
    }
    return $this->layoutBuilderIpe->hashLayout($entity) != $original_hash;
  }

  /**
   * See if the given entity has been submitted via Layout Builder IPE.
   *
   * @param \Drupal\Core\Entity\EntityInterface
   *   The entity object.
   *
   * @return bool
   */
  public function hasEntityChanges($entity) {
    $original_hash = \Drupal::request()->get('layout_builder_ipe_entity_hash');
    if (empty($original_hash)) {
      return FALSE;
    }
    return $this->layoutBuilderIpe->hashEntity($entity) != $original_hash;
  }

}